package com.simibubi.create.modules.contraptions.components.flywheel;

import com.simibubi.create.foundation.utility.Lang;
import com.simibubi.create.modules.contraptions.base.HorizontalKineticBlock;

import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.Blocks;
import net.minecraft.item.BlockItemUseContext;
import net.minecraft.state.EnumProperty;
import net.minecraft.state.StateContainer.Builder;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.Direction;
import net.minecraft.util.Direction.Axis;
import net.minecraft.util.IStringSerializable;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockReader;
import net.minecraft.world.IWorldReader;
import net.minecraft.world.World;

public class FlywheelBlock extends HorizontalKineticBlock {

	public static EnumProperty<ConnectionState> CONNECTION = EnumProperty.create("connection", ConnectionState.class);

	public FlywheelBlock() {
		super(Properties.from(Blocks.GOLD_BLOCK));
		setDefaultState(getDefaultState().with(CONNECTION, ConnectionState.NONE));
	}

	@Override
	protected void fillStateContainer(Builder<Block, BlockState> builder) {
		super.fillStateContainer(builder.add(CONNECTION));
	}

	@Override
	public TileEntity createTileEntity(BlockState state, IBlockReader world) {
		return new FlywheelTileEntity();
	}

	@Override
	public BlockState getStateForPlacement(BlockItemUseContext context) {
		Direction preferred = getPreferredHorizontalFacing(context);
		if (preferred != null)
			return getDefaultState().with(HORIZONTAL_FACING, preferred.getOpposite());
		return this.getDefaultState().with(HORIZONTAL_FACING, context.getPlacementHorizontalFacing());
	}

	@Override
	protected boolean hasStaticPart() {
		return true;
	}

	public static boolean isConnected(BlockState state) {
		return getConnection(state) != null;
	}

	public static Direction getConnection(BlockState state) {
		Direction facing = state.get(HORIZONTAL_FACING);
		ConnectionState connection = state.get(CONNECTION);

		if (connection == ConnectionState.LEFT)
			return facing.rotateYCCW();
		if (connection == ConnectionState.RIGHT)
			return facing.rotateY();
		return null;
	}

	public static void setConnection(World world, BlockPos pos, BlockState state, Direction direction) {
		Direction facing = state.get(HORIZONTAL_FACING);
		ConnectionState connection = ConnectionState.NONE;

		if (direction == facing.rotateY())
			connection = ConnectionState.RIGHT;
		if (direction == facing.rotateYCCW())
			connection = ConnectionState.LEFT;

		world.setBlockState(pos, state.with(CONNECTION, connection), 18);
	}

	@Override
	public boolean hasShaftTowards(IWorldReader world, BlockPos pos, BlockState state, Direction face) {
		return face == state.get(HORIZONTAL_FACING).getOpposite();
	}

	@Override
	public Axis getRotationAxis(BlockState state) {
		return state.get(HORIZONTAL_FACING).getAxis();
	}

	public enum ConnectionState implements IStringSerializable {
		NONE, LEFT, RIGHT;

		@Override
		public String getName() {
			return Lang.asId(name());
		}
	}

}
